/*
Copyright (c) 2018 Vereign AG [https://www.vereign.com]

This is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package api.test.rest.tsa.policy;

import api.test.core.BaseStepDefinitions;
import api.test.rest.RestGeneralStepDefinitions;
import api.test.rest.RestSessionContainer;
import com.mongodb.MongoException;
import com.mongodb.client.MongoClient;
import com.mongodb.client.MongoClients;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.result.DeleteResult;
import core.DataContainer;
import core.Request;
import core.Response;
import core.RestClient;
import cucumber.api.java.en.And;
import cucumber.api.java.en.Given;
import cucumber.api.java.en.Then;
import cucumber.api.java.en.When;
import cucumber.api.java8.Th;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bson.Document;
import org.bson.conversions.Bson;
import org.bson.types.ObjectId;

import static api.test.core.GeneralStepDefinitions.the_field_has_the_value_;
import static com.mongodb.client.model.Filters.eq;

public class PolicyStepDefinitions extends BaseStepDefinitions {

    private static final Logger logger = LogManager.getLogger(RestGeneralStepDefinitions.class.getSimpleName());
    public static final String mongoConnection = "mongodb://root:root@localhost:27017";
    RestSessionContainer restSessionContainer;
    Request currentRequest;

    public PolicyStepDefinitions(RestSessionContainer restSessionContainer, Request currentRequest, DataContainer dataContainer) {
        super(dataContainer);
        this.restSessionContainer = restSessionContainer;
        this.currentRequest = currentRequest;
    }

    @When("^I upload a policy \\{(.*)\\} to repository$")
    public void iUploadThePolicyToRepository(String policy) throws Throwable {
        MongoClient client = MongoClients.create(mongoConnection);
        MongoDatabase database = client.getDatabase("policy");
        MongoCollection<Document> policies = database.getCollection("policies");
        Document test = new Document("_id", new ObjectId())
                .append("filename", "example_2.0.rego")
                .append("name", policy)
                .append("group", "testable")
                .append("version", "2.0")
                .append("locked", false)
                .append("rego", "package testable."+ policy +"\n" +
                        "default allow = false\n" +
                        "allow {input.message == \"hello world\"}\n" +
                        "taskID := \"0123456\"");
        policies.insertOne(test);
    }

    @Given("I upload a DID resolver policy to repository")
    public void iUploadADIDResolverPolicyDidResolverToRepository() {
        MongoClient client = MongoClients.create(mongoConnection);
        MongoDatabase database = client.getDatabase("policy");
        MongoCollection<Document> policies = database.getCollection("policies");
        Document test = new Document("_id", new ObjectId())
                .append("filename", "resolve_1.0.rego")
                .append("name", "resolve")
                .append("group", "example")
                .append("version", "1.0")
                .append("locked", false)
                .append("rego", "package example.resolve\n" +
                        "data = did.resolve(input.did)\n" +
                        "taskID := \"01234567\"");
        policies.insertOne(test);
    }

    @And("^I execute the policy \\{(.*)\\}$")
    public void iExecuteThePolicy(String path) throws Throwable {
        currentRequest.setPath(path);
        Response response = RestClient.post(currentRequest);
        addRequest(currentRequest);
        addResponse(response);
    }

    @When("^I lock the policy \\{(.*)\\}$")
    public void iLockThePolicy(String path) throws Throwable {
        currentRequest.setPath(path);
        Response response = RestClient.post(currentRequest);
        addRequest(currentRequest);
        addResponse(response);
    }

    @When("^I unlock the policy \\{(.*)\\}$")
    public void iUnlockThePolicy(String path) throws Throwable {
        currentRequest.setPath(path);
        Response response = RestClient.delete(currentRequest);
        addRequest(currentRequest);
        addResponse(response);
    }

    @And("^delete policy \\{(.*)\\} from repository$")
    public void deletePolicyTestFromRepository(String policy) {
        MongoClient client = MongoClients.create(mongoConnection);
        MongoDatabase database = client.getDatabase("policy");
        MongoCollection<Document> policies = database.getCollection("policies");
        Bson query = eq("name", policy);
        try {
            DeleteResult result = policies.deleteOne(query);
            System.out.println("Deleted document count: " + result.getDeletedCount());
        } catch (MongoException me) {
            System.err.println("Unable to delete due to an error: " + me);
        }
    }
}