#Copyright (c) 2018 Vereign AG [https://www.vereign.com]
#
#This is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as
#published by the Free Software Foundation, either version 3 of the
#License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.
#
#You should have received a copy of the GNU Affero General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

#http://localhost:8081/policy/:group/:name/:version/evaluation
#Author: Georgi Michev georgi.michev@vereign.com

@rest @all @tsa @policy
Feature: API -TSA - Policy - :group/:name/:version/evaluation POST
  As user
  I want to evaluate the policy
  So I am able to execute it in the future

  Acceptance criteria:
  - HTTP endpoint to evaluate the policy
  - example policy created and committed to Git repo
  - Green test based on example committed to the system

  Background:
    Given we are testing the TSA Policy Api

  Scenario: TSA - Evaluate policy synchronously - Positive
    Given I upload a policy {test1} to repository
    Then I load the REST request {Policy.json} with profile {successful_evaluate}
    And I execute the policy {/testable/test1/2.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_Evaluate_schema.json} REST schema
    And the field {allow} has the value {true}
    And the field {taskID} has the value {0123456}
    And delete policy {test1} from repository

  Scenario: TSA - DID resolution - Positive
    Given I upload a DID resolver policy to repository
    Then I load the REST request {Policy.json} with profile {didResolve_evaluate}
    And I execute the policy {/example/resolve/1.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_EvaluateDID_schema.json} REST schema
    And the field {data.didDocument.id} has the value {did:indy:idunion:BDrEcHc8Tb4Lb2VyQZWEDE}
    And the field {taskID} has the value {01234567}
    And delete policy {resolve} from repository

  @negative
  Scenario: TSA - Evaluate policy with incorrect value - Negative
    Given I upload a policy {test6} to repository
    Then I load the REST request {Policy.json} with profile {unsuccessful_evaluate}
    And I execute the policy {/testable/test6/2.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_Evaluate_schema.json} REST schema
    And the field {allow} has the value {false}
    And the field {taskID} has the value {0123456}
    And delete policy {test6} from repository

  @negative
  Scenario: TSA - Evaluate policy with missing body- Negative
    Given I upload a policy {test7} to repository
    And I execute the policy {/testable/test7/2.0/evaluation}
    Then the status code should be {400}
    And the response is valid according to the {Policy_Evaluate_negative_schema.json} REST schema
    And the field {message} has the value {missing required payload}
    And delete policy {test7} from repository

  @negative @wip
  Scenario: TSA - Evaluate missing policy - Negative
    Given I execute the policy {/testable/test0/2.0/evaluation}
    Then the status code should be {400}
    And the response is valid according to the {Policy_Evaluate_negative_schema.json} REST schema
#    And the field {message} has the value {policy not found}

  @negative
  Scenario: TSA - DID resolution - Negative
    Given I upload a DID resolver policy to repository
    Then I load the REST request {Policy.json} with profile {didResolve_missing_method}
    And I execute the policy {/example/resolve/1.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_EvaluateDID_negative_schema.json} REST schema
    And the field {data.didResolutionMetadata.error} has the value {notFound}
    And delete policy {resolve} from repository
